# Qualité

> Quiconque cherche à circonscrire une discipline en lui imposant un cadre ne cherche généralement qu’à protéger son œuvre, et il le fait en perpétuant les standards qui ont permis son émergence. Celui-ci essaie de convaincre les générations à venir qu’elles doivent suivre les règles qu’il a édictées si elles comptent parvenir à l’excellence. Mais, comme disait Charles Bukowski, « il est quatre heures et demie du matin, il sera toujours quatre heures et demie du matin… ».
> 
> Nous sommes à ce point focalisés sur le chemin que nous nous efforçons de suivre, en voulant toujours tout faire au mieux, armés d’une dévotion sans faille pour notre discipline ==alors que les œillères de la peur nous empêchent d’envisager des terrains inconnus,== que nos yeux restent fixés sur cette route, sur ces mains qui prennent appui sur des genoux. Et nous ne réalisons pas que nous ne faisons que suivre les règles promulguées par un homme qui a un jour disputé une course contre des chevaux, ou par un autre qui s’est frotté à un sommet de plus de huit mille mètres sans oxygène, ou par un autre encore qui a décidé de laisser chez lui ses pitons, ses cordes et la sécurité pour ne faire qu’un avec les murs à escalader. Nous suivons les lois de ceux qui en ont enfreint de plus anciennes.
> 
> <cite>*Au-delà des sommets*, Kilian Jornet</cite>

On parlait de code et de qualité avec [Thomas](https://thom4.net/). Je lui faisais part de ma frustration vis-à-vis d’un code qui n’avait pas été écrit par moi et que je trouvais problématique. En creusant un peu (merci !), je réalise que ce qui coince est au niveau de la pérennité et de la transmission. Et j’ai aussi conscience de produire moi-même du code qui serait difficile à reprendre par d’autres personnes n’ayant pas les mêmes aspirations/compétences.

En Python, on a la chance de pouvoir automatiser certaines conversions/vérifications qui tendent à aller vers une certaine uniformisation (et donc universalité ?) : [black](https://github.com/psf/black), [flake8](https://flake8.pycqa.org/), [isort](https://pypi.org/project/isort/) ou [mypy](https://mypy.readthedocs.io/) par exemple.

Pour aller plus loin, le code en lui-même n’est peut-être pas si critique, mais ce que l’on a appris en le concevant et l’utilisant l’est bien davantage. C’est cette transmission qu’il est important de rendre possible au sein de l’équipe. Outiller la base commune est un moyen de **plus rapidement** passer à l’étape de partage des concepts importants/métiers, en ce souciant moins de la forme.

Et peut-être au contraire, que cette **vitesse** acquise nous empêche d’échanger sur des concepts importants ? Des envies différentes ? Des choix à côté desquels on peut passer par manque d’attention.

> [en] Tradition (n.): Peer pressure from dead people.
>
> <cite>*Lu plusieurs fois sur masto*</cite>

---

> 🦋 Depuis quelques années, j’essaie d’écrire un code le plus direct possible.
>
> Mon objectif : diminuer au maximum ma charge cognitive.
> 
> <cite>*[Keep it simple, stupid le plus longtemps possible](https://sklein.xyz/garden/020-keep-it-simple-stupid-le-plus-longtemps-possible/)* ([cache](/david/cache/2023/646ebfa25432bc6b653e524b52d25c9a/))</cite>


> [en] ⛵️ Many of the tools that we thought we could rely on broke down, whether it is Apple products, or software that require subscription services, DRM, etc. As an artist you spend time developing a skill, ==you become a Photoshop illustrator.== When your connection to the internet fails and that the software locks up, that skill that you thought was yours was actually entirely owned by someone, and can be taken away.
>
> Even though we’ve been paying for this sort of software for years, the moment that you can’t have access to authenticate yourself that skill is gone. We didn’t expect this, it scared us.
>
> <cite>*[weathering software winter](https://100r.co/site/weathering_software_winter.html)* ([cache](/david/cache/2023/c564325b28ec854b6b098950df5c8c8f/))</cite>

> [en] 🔎 It’s important to remember concepts and high level approaches, but don’t worry about remembering the details. You can always look that stuff up when you need it.
> 
> <cite>*[You don’t have to remember everything to be a good programmer](https://gomakethings.com/you-dont-have-to-remember-everything-to-be-a-good-programmer/)* ([cache](/david/cache/2023/2f4b8ec8f6fd2cbc48a04dcac2cc7d24/))</cite>

#accompagnement #qualité #technique