title:      Formatage des chaînes de caractères en Python
slug:       formatage-des-chaines-de-caracteres-en-python
date:       2006-01-21 20:59:27
type:       post
vignette:   images/logos/bonnes_pratiques_python.png
contextual_title1:  Bonnes pratiques et astuces Python
contextual_url1:    20080511-bonnes-pratiques-et-astuces-python
contextual_title2:  Benchmarks map, filter vs. list-comprehensions
contextual_url2:    20061025-benchmarks-map-filter-vs-list-comprehensions
contextual_title3:  Python : lisibilité vs simplicité
contextual_url3:    20060425-python-et-underscore

<p>À l'intersection entre bonnes pratiques et optimisation comment afficher des chaînes de caractère en Python&nbsp;? Privillégier la lisibilité ou l'optimisation&nbsp;? Les deux mon capitaine&nbsp;!</p>

<p>Il existe deux méthodes assez classiques en Python pour formater des chaînes de caractère, à partir de <strong>signes plus (+)</strong> ou de <strong>%s</strong>. Considérons les chaînes de caractères suivantes&nbsp;:</p>


<pre>title = "optimisation" * 1000
cat = "python" * 1000
main_title = "biologeek" * 1000</pre>


<p>Si l'on utilise des <strong>signes plus (+)</strong>&nbsp;:</p>


<pre>def foo1():
    page = ""
    for x in range(500):
        page += "&lt;title&gt;" + title + " - " + cat + " - " + main_title + "&lt;/title&gt;"
    return page</pre>


<p>Cette fonction s'exécute en <strong>5,91 secondes</strong>, si l'on utilise des <strong>%s</strong>&nbsp;:</p>


<pre>def foo2():
    page = ""
    for x in range(500):
        page += "&lt;title&gt;%s - %s - %s&lt;/title&gt;" % (title, cat, main_title)
    return page</pre>


<p>Elle s'exécute dans le même temps. À choisir, beaucoup préfèrent la seconde (notamment en raison du lourd héritage d'autre langages préhistoriques...) mais heureusement il existe en python un moyen de faire encore plus lisible&nbsp;: utiliser des dictionnaires.</p>


<pre>def foo3():
    dic = {
        "title" : title,
        "cat" : cat,
        "main_title" : main_title
    }
    page = ""
    for x in range(500):
        page += "&lt;title&gt;%(title)s - %(cat)s - %(main_title)s&lt;/title&gt;" % dic
    return page</pre>


<p>Le temps d'exécution est ici aussi identique et si vous ne voulez pas créer de dictionnaire supplémentaire, il est possible de mettre directement le dictionnaire en argument&nbsp;:</p>


<pre>page += "&lt;title&gt;%(title)s - %(cat)s - %(main_title)s&lt;/title&gt;" %\
{"title":title, "cat":cat, "main_title":main_title}</pre>


<p>Enfin il existe une dernière méthode utilisant les fonctions intégrées (pour en savoir plus, je vous renvoie sur l'excellent <a href="http://diveintopython.adrahon.org/html_processing/locals_and_globals.html">Dive into Python</a>).</p>


<pre>def foo4():
    title = "optimisation" * 1000
    cat = "python" * 1000
    main_title = "biologeek" * 1000
    page = ""
    for x in range(500):
        page += "&lt;title&gt;%(title)s - %(cat)s - %(main_title)s&lt;/title&gt;" % locals()
    return page</pre>


<p>Mais elle met <strong>6,07 secondes</strong> à s'exécuter et ce temps augmente avec le nombre de variables locales donc c'est moins performant... mais tellement pratique :-).</p>


<p>Conclusion, si vous voulez concilier les deux, à savoir optimisation et lisibilité, je vous conseille d'utiliser des dictionnaires pour formater vos chaînes de caractères et si vous n'avez pas nécessairement besoin d'aller vite <strong>locals()</strong> se révèle être très pratique (encore une fois faites des tests !).</p>


<p>Je vous rappelle qu'un billet récapitule l'ensemble des <a href="https://larlet.fr/david/biologeek/archives/20060121-bonnes-pratiques-de-la-programmation-en-python/">bonnes pratiques et optimisations en Python</a>.</p>
