title:      Optimisation des chaînes de caractères en Python
slug:       optimisation-des-chaines-de-caracteres-en-python
date:       2006-01-21 20:13:01
type:       post
vignette:   images/logos/bonnes_pratiques_python.png
contextual_title1:  Bonnes pratiques et astuces Python
contextual_url1:    20080511-bonnes-pratiques-et-astuces-python
contextual_title2:  Benchmarks map, filter vs. list-comprehensions
contextual_url2:    20061025-benchmarks-map-filter-vs-list-comprehensions
contextual_title3:  Python : lisibilité vs simplicité
contextual_url3:    20060425-python-et-underscore

<p>Dans quelles situations utiliser les chaînes de caractère&nbsp;? Pourquoi pas des listes&nbsp;? Et les list-comprehension dans tout ça&nbsp;? Réponses en tests, c'est plein de strings mais ne vous inquiétez pas, rien de sexuel ;)</p>

<p>Considérons une chaîne de caractères assez conséquente&nbsp;:</p>


<pre>strings = ["tagada"]*1000000 + ["tsouintsouin"]*1000000</pre>


<p>Puis une fonction concaténant tous les tagada&nbsp;:</p>


<pre>def foo1():
    string_final = ""
    for string in strings:
        if not "tsouin" in string:
            string_final += string
    return string_final</pre>


<p>Nous obtenons une fonction qui met <strong>4,44 secondes</strong> pour renvoyer une longue chaîne. Pas vraiment rapide, je suis sûr qu'on pourrait aller plus vite avec des listes&nbsp;:</p>


<pre>def foo2():
    string_final = []
    for string in strings:
        if not "tsouin" in string:
            string_final.append(string)
    return "".join(string_final)</pre>


<p>Cette fois ce sont <strong>4,65 secondes</strong> qui sont nécessaires pour effectuer la même opération... l'ajout d'un élément à une liste est donc très coûteux en temps aussi&nbsp;! Et c'est loin des «&nbsp;<a href="http://wikipython.flibuste.net/moin.py/BonnesPratiques#head-1c5e141b2f386f3757d49d2fbce14170aed3a687">centaines de fois</a> » plus rapide qui peuvent être annoncées. C'est d'ailleurs la raison pour laquelle il existe les comprehension-lists&nbsp;:</p>


<pre>def foo3():
    return "".join([string for string in strings if not "tsouin" in string])</pre>


<p>Qui ne mettent dans ce cas là que <strong>3,25 secondes</strong>&nbsp;! Voila qui fait réfléchir lors de votre prochaine implémentation ;-). Mais au fait s'il est aussi coûteux en temps d'ajouter des éléments à une liste, pourquoi ne pas créer une chaîne de caractères que l'on découpe ensuite en une liste ?! Farfelu&nbsp;? Vérifions&nbsp;:</p>


<pre>def foo4():
    string_final = ""
    for string in strings:
        if not "tsouin" in string:
            string_final += string + "separateur"
    return string_final.split("separateur")

def foo5():
    string_final = []
    for string in strings:
        if not "tsouin" in string:
            string_final.append(string)
    return string_final</pre>


<p>La première fonction renvoie une liste en <strong>17,8 secondes</strong>, la seconde en <strong>4,31 secondes</strong>, en effet c'était farfelu. Et avec une list-comprehension&nbsp;? Théoriquement encore plus rapide que précédemment puisque qu'il y a un <strong>join()</strong> en moins, je vous laisse vérifier.</p>


<p>Conclusion&nbsp;: utilisez les list-comprehension au maximum lorsque c'est possible quitte à faire des <strong>[foo(string) for string in strings]</strong> s'il y a de gros post-traitements de <strong>string</strong>. Quant à l'utilisation de concaténation de chaînes de caractères contre l'ajout à une liste, le temps où les listes étaient beaucoup plus rapides est apparemment révolu, choisissez donc en fonction du type de données et non de la rapidité...</p>


<p><strong>[Edit/Erratum]</strong>&nbsp;: un billet suivant explique une <a href="https://larlet.fr/david/biologeek/archives/20060127-optimisation-des-chaines-de-caracteres-en-python-le-retour/">nouvelle méthode d'utilisation des listes plus rapide que les chaînes de caractères</a>.</p>


<p>Je vous rappelle qu'un billet récapitule l'ensemble des <a href="https://larlet.fr/david/biologeek/archives/20060121-bonnes-pratiques-de-la-programmation-en-python/">bonnes pratiques et optimisations en Python</a>.</p>
