title:      Optimisation des chaînes de caractères en Python : le retour !
slug:       optimisation-des-chaines-de-caracteres-en-python-le-retour
date:       2006-01-27 12:37:20
type:       post
vignette:   images/logos/bonnes_pratiques_python.png
contextual_title1:  Bonnes pratiques et astuces Python
contextual_url1:    20080511-bonnes-pratiques-et-astuces-python
contextual_title2:  Benchmarks map, filter vs. list-comprehensions
contextual_url2:    20061025-benchmarks-map-filter-vs-list-comprehensions
contextual_title3:  Python : lisibilité vs simplicité
contextual_url3:    20060425-python-et-underscore

<p>Dans les <a href="https://larlet.fr/david/biologeek/archives/20060121-optimisation-des-chaines-de-caracteres-en-python/">épisodes précédents</a>, je m'étonnais de voir des concaténations de chaînes de caractères être plus rapide que des remplissages de listes. Depuis je cogite car il est indiqué un peu partout qu'il faut privillégier les listes. Et j'ai fini par trouver une réponse :-).</p>

<p>Considérons la même chaîne de caractères&nbsp;:</p>


<pre>strings = ["tagada"]*1000000 + ["tsouintsouin"]*1000000</pre>


<p>Et la même fonction <strong>foo2()</strong>&nbsp;:</p>


<pre>def foo2():
    string_final = []
    for string in strings:
        if not "tsouin" in string:
            string_final.append(string)
    return "".join(string_final)</pre>


<p>Donc cette fonction met environ <strong>4,70 secondes</strong> à créer cette énorme chaîne de caractères. Ce que j'avais alors oublié de tester, ce sont les alias. En effet, les fonctions de type <strong>list.append</strong> sont réévaluées à chaque itération en python, ce qui fait perdre un temps fou, la preuve en chiffres&nbsp;:</p>


<pre>def foo3():
    string_final = []
    app = string_final.append
    for string in strings:
        if not "tsouin" in string:
            app(string)
    return "".join(string_final)</pre>


<p>Et là on tombe à <strong>3,50 secondes</strong>&nbsp;! Ainsi les listes peuvent être plus rapides que les concaténations de chaînes de caractères, encore faut-il pouvoir créer un alias...</p>


<p>Bon et puisque j'y suis, une autre petite astuce d'optimisation&nbsp;: on peut lire dans les <a href="http://www.zope.org/Wikis/DevSite/Projects/ComponentArchitecture/CodeOptimization">astuces d'optimisation de Zope</a> qu'il vaut mieux utiliser les méthodes du module string plutôt que des portions de listes. Bon dit comme ça on comprend pas trop de quoi je parle alors un petit exemple pour clarifier tout ça&nbsp;:</p>


<pre>def foo4():
    string_final = ""
    for string in strings:
        if string[:5] == "tagad":
            string_final += string
    return string_final
    
def foo5():
    string_final = ""
    for string in strings:
        if string.startswith("tagad"):
            string_final += string
    return string_final</pre>


<p>La méthode <strong>foo4()</strong> s'exécute en <strong>7,2 secondes</strong> et <strong>foo5()</strong> en <strong>12 secondes</strong>... tiens c'est le contraire qui est obtenu ici, me serais-je trompé&nbsp;? Je revérifie et non j'ai bien ces temps là d'exécution, soit presque le double lorsque l'on utilise les méthodes du module string.</p>


<p>Première conclusion&nbsp;: vérifiez toujours les astuces d'optimisation qu'on vous propose (ceci est valable pour celles-ci aussi) car elle peuvent évoluer selon les versions de Python ou du contexte par exemple.</p>


<p>Seconde conclusion&nbsp;: les listes finalement c'est pas si mal pour traiter les chaînes de caractères :-).</p>


<p>Je vous rappelle qu'un billet récapitule l'ensemble des <a href="https://larlet.fr/david/biologeek/archives/20060121-bonnes-pratiques-de-la-programmation-en-python/">bonnes pratiques et optimisations en Python</a>.</p>
