title:      ★ Pour ne plus être en REST, comprendre cette architecture
slug:       pour-ne-plus-etre-en-rest-comprendre-cette-architecture
date:       2007-04-13 08:58:50
type:       post
vignette:   images/logos/rest_softies.png
contextual_title1:  ★ De l&#39;OpenData au LinkedData : exemple de nosdonnees.fr
contextual_url1:    20101130-de-lopendata-au-linkeddata-exemple-de-nosdonneesfr
contextual_title2:  ★ Django-ROA, pour une architecture orientée ressources
contextual_url2:    20090526-django-roa-pour-une-architecture-orientee-ressources
contextual_title3:  ★ Le Web Sémantique ou l&#39;importance des données liées 
contextual_url3:    20081117-le-web-semantique-ou-limportance-des-donnees-liees

<p>Depuis quelques mois, <del>j'apprends</del> j'essaye d'apprendre <abbr title="Representational state transfer">REST</abbr> et j'ai lu de nombreuses pages sans pour autant trouver d'explication simple, à la portée de tous. Du coup quand j'essaye d'expliquer les mécanismes et les avantages professionnellement, c'est pas toujours convaincant... et je voulais justement écrire un petit billet pour mettre tout ça au clair. Et puis je suis récemment tombé sur <a href="http://softiesonrails.com/tags/rest">une série d'explications par Softies on Rails</a> se basant sur des exemples concrets. En voici la traduction chronologique.</p>

<h2>Comprendre les ressources</h2>


<p>D'une certaine manière, apprendre REST fut très difficile pour moi. Je ne sais pas vraiment pourquoi. Peut-être car j'ai passé une bonne partie de ma vie à développer des applications client et non des applications web. Peut-être est-ce parce que (à lire rapidement) j'ai commencé à développer pour le web tout en apprenant Ruby tout en apprenant Rails et que j'ai ensuite été à RailsConf et tout le monde était excité par REST et j'approuvais pour ne pas passer pour un imbécile mais honnêtement je ne savais pas de quoi il retournait...</p>


<p><strong>Si vous essayez de comprendre REST, cette série est faite pour vous.</strong></p>


<blockquote><p>Les experts en <abbr title="HyperText Transfer Protocol">HTTP</abbr>, <abbr title="HyperText Markup Language">HTML</abbr> et REST pourraient chercher la petite bête avec ma terminologie simplifiée me permettant d'aller droit au but. Si c'est votre cas, je ne veux pas en entendre parler, car cette série ne vous est pas destinée de toute façon :-)</p></blockquote>


<h3>Premier point</h3>


<p>Commençons par le commencement. Comment fonctionne un navigateur&nbsp;? Avant d'essayer de développer des sites avec Rails, je pensais que ça marchait ainsi&nbsp;:</p>

<ul>
<li>je tape une url dans la barre d'adresse ou je clic sur un lien&nbsp;;</li>
<li>le navigateur fait une «&nbsp;requête http » (quoi que cela puisse signifier) et obtient du code HTML en réponse&nbsp;;</li>
<li>le navigateur interprète ce HTML pour afficher la page.</li>
</ul>

<p>Et c'est à peu près tout. Et je ne me suis jamais demandé comment les formulaires fonctionnaient. Je me disais juste que ça devait être une variante de la précédente description.</p>


<p>En réalité, le protocole HTTP décrit les instructions détaillées de la façon dont un navigateur doit envoyer ses requêtes au serveur. HTTP est totalement différent du HTML, qui est le langage choisit pour représenter le contenu d'une page. HTTP permet au navigateur de récupérer divers types d'informations à partir du serveur, dont la simple récupération d'une page HTML ou la soumission d'un formulaire. En fait, HTTP permet d'effectuer huit types de requêtes différentes sur le serveur. Les deux requêtes les plus connues sont les suivantes&nbsp;:</p>
<ul>
<li><strong>une requête «&nbsp;GET »&nbsp;: récupère le contenu d'une ressource sur le web, la ressource est uniquement identifié par une <abbr title="Uniform Resource Locator">URL</abbr></strong>&nbsp;;</li>
<li><strong>une requête «&nbsp;POST »&nbsp;: envoie des données à une URL afin de créer une nouvelle ressource</strong>.</li>
</ul>

<h3>Deuxième point</h3>


<p>Remarquez comme j'ai glissé le mot «&nbsp;ressource » alors que vous ne l'attendiez pas. Ça a été mon premier mal de tête lorsque j'ai essayé de comprendre REST&nbsp;: <strong>penser au web comme un ensemble de ressources, pas de «&nbsp;pages web »</strong>. Qu'est-ce que je veux dire par là&nbsp;?</p>


<p>Hier, je suis allé sur amazon.com et j'ai consulter quelques produit que je voudrais acheter. J'ai aussi lu quelques articles de Wikipedia. J'ai parcouru quelques news de TechCrunch et j'ai ensuite vérifier les scores de D1 pour constater à quel point Lyon est loin devant.</p>


<p>Si vous voulez comprendre REST, vous devez arrêter de penser à tout ça en termes de pages web. Prenons l'exemple d'un article de Wikipedia. Ce n'est pas seulement une page web, c'est une ressource - dans notre exemple un biographie d'Archimède. J'utilise un navigateur web pour accéder à cette ressource, le navigateur récupère donc une représentation HTML de cette ressource, car afficher du HTML est la seule possibilité de représentation de ces ressources par un navigateur.</p>


<p>Je sais que c'est étrange au début. L'article sur Archimède n'est pas une page web&nbsp;? Non. J'ai demandé la représentation HTML d'une ressource appartenant à Wikipedia, une ressource pouvant être identifiée par l'<abbr title="Universal Resource Identifier">URI</abbr> «&nbsp;//fr.wikipedia.org/wiki/Archimède ». Wikipedia aurait pu choisir de représenter cette ressource de beaucoup de façons - avec un <acronym>PDF</abbr>, ou peut-être une image .jpg. C'est même peut-être déjà possible. Mais Firefox utilise une requête GET qui spécifie que cette ressource doit être obtenue à partir du serveur sous forme de HTML, donc c'est ce que j'ai.</p>


<p>Un autre exemple&nbsp;: ma réservation d'avion est une ressource détenue par Air France. Ils me permettent d'accéder à cette ressource de diverses manières&nbsp;: via une page HTML, mais ils peuvent aussi envoyer les informations de ma réservation sur mon téléphone portable. Ou ils peuvent me l'envoyer par mail en texte brut. Ou je peux les appeler avec un outil obsolète et ils vont me le dire de vive voix. <strong>Une même ressource, plusieurs représentations.</strong></p>


<p>J'espère que c'est clair maintenant&nbsp;: ma réservation d'avion n'est pas qu'une page web. C'est une vraie ressource qui (heureusement pour moi) peut être affichée en HTML lorsque je le désire, et je peux utilise mon navigateur pour demander cette représentation et l'afficher sur mon écran.</p>


<h3>Troisième point</h3>


<p>Ok, maintenant que vous avez accepté le fait que <strong>le web est une gigantesque collection de ressources qui peuvent être servies de différentes manières</strong>, et qu'un affichage HTML est juste l'une d'entre elles, il y a maintenant un dernier concept que je souhaite aborder. Les ressources ne sont pas uniquement des simples articles biographiques, ou des réservations d'avions, ou des résultats sportifs. <strong>Certaines ressources sont des collections d'autres ressources.</strong> Une liste de vacances peut être une ressource. Une liste d'amis peut être une ressource. Une série de billets de blog peut être une ressource.</p>


<p>Maintenant que vous avez compris ce que sont les ressources, nous allons maintenant nous intéresser à la façon dont HTTP permet la création, la récupération, la mise à jour et la suppression des ressources (<em><abbr title="Note du traducteur">NdT</abbr>&nbsp;: bon normalement c'est <strong>crud</strong> pour <strong>c</strong>reate, <strong>r</strong>etrieve, <strong>u</strong>pdate et <strong>d</strong>elete</em>).</p>


<h2>Des millions d'API</h2>


<h3>Analyse orientée objet et design</h3>


<p>Si vous programmez en orienté objet depuis un certain temps, vous avez probablement appris à procéder de la façon suivante&nbsp;:</p>

<ul>
<li>écrire une description de votre problématique - ce que votre programme est censé faire.</li>
<li>les noms sont vos classes.</li>
<li>les verbes sont les méthodes de vos classes.</li>
<li>permettre à vos noms d'appeler les méthodes d'autres noms pour qu'ils accomplissent des tâches.</li>
</ul>

<p>Ça semble bon. C'est ce que je fais en programmation depuis un moment déjà. Ce type de programmation par nom-verbe est souvent appelé style <a href="http://fr.wikipedia.org/wiki/Remote_procedure_call">RPC</a>. Je ne sais pas en quoi c'est proche de l'accès à distance mais je sais juste que le style RPC est la façon «&nbsp;classique » de développer des logiciels avec de l'orienté objet. C'est une très bonne manière pour développer des logiciels mais ce n'est pas la façon dont le web a été pensé. Pourquoi ça&nbsp;?</p>


<p>Imaginez-vous en 1992 (ou juste avant que le web soit tel que vous le connaissez aujourd'hui). Et supposez 3 équipes différentes qui développent 3 logiciels&nbsp;: un pour acheter des livres, un pour acheter des billets d'avion et un autre qui permet d'obtenir des photographies aériennes de n'importe quelle distance de la Terre. Elles suivent toutes les 3 les techniques de modélisation orientée objet classiques dont nous avons parlé pour développer leurs applications. Et pensant qu'un tiers voudra probablement un jour s'interfacer avec leur logiciel, elles vont aussi implémenter leurs propres API.</p>


<p>Maintenant supposons que ce soit votre boulot d'écrire un outil qui soit une interface de ces 3 applications.</p>


<p>Ugh. Vous allez devoir apprendre 3 API différentes. Et je parie que vous voudriez des boutons qui correspondent directement aux méthodes que vous appelez pour chaque API. Ça semble logique, non&nbsp;? Je veux dire, comment feriez-vous autrement&nbsp;? Ok, on ne parle que de 3 API et vous connaissez de bon ouvrages qui ont des design patterns qui vont vous permettre de réduire cette complexité (et de rester aimable avec vos collègues par la même occasion).</p>


<h3>Et pour un million d'API&nbsp;?</h3>


<p>Revenons au navigateur. Lorsque vous l'installez sur votre ordinateur, il n'a aucune idée des sites que vous allez vouloir visiter. Par contre, il peut communiquer avec n'importe lequel de ces sites. Mais pas seulement, vous pouvez réaliser des actions sur ces sites&nbsp;: vous pouvez acheter de la musique, réserver des billets d'avion et avoir des vues satellites de votre maison.</p>


<p>Est-ce magique&nbsp;? Réfléchissez un moment à l'impossibilité d'une telle approche si chaque site avait sa propre API. Pour acheter un livre sur Amazon, le navigateur aurait dû savoir appeler la méthode Amazon.Acheter(). Et il aurait dû appeler la méthode AirFrance.VérifierVols() lorsque j'ai cliqué sur le bouton «&nbsp;Vérifier les vols ». Vous ne pouvez tout simplement pas développer un outil qui permette d'appeler des millions d'API différentes.</p>


<p><strong>C'est la raison pour laquelle le web n'a pas été designé avec une architecture RPC. Il a été designé avec une architecture REST.</strong></p>


<h3>Une architecture orientée ressource</h3>


<p>D'après Wikipedia, <a href="http://fr.wikipedia.org/wiki/REST">REST</a> signifie Representational State Transfer. Bon... peu importe. Cela signifie simplement qu'en lieu et place des noms avec des verbes arbitraires associés, chaque nom (que nous appellerons maintenant ressource) aura le même nombre fini de verbes. <strong>En d'autres termes, chaque ressource aura la même API</strong>. Les méthodes clés de cette API autorisent chaque client à&nbsp;:</p>
<ul>
<li>avoir une vue en lecture seule de la ressource&nbsp;;</li>
<li>créer une nouvelle ressource&nbsp;;</li>
<li>mettre à jour les attributs d'une ressource&nbsp;;</li>
<li>supprimer une ressource.</li>
</ul>

<p>Attendez&nbsp;! Laquelle de ces méthode permet «&nbsp;d'acheter ce livre »&nbsp;? Laquelle permet de «&nbsp;chercher les vols entre Paris et Marseille mardi prochain »&nbsp;? Laquelle me permet de zoomer d'une ville à une rue sur une carte&nbsp;? Et laquelle permet à un utilisateur de se connecter à un site&nbsp;?</p>


<h2>Architecture RESTful</h2>


<h3>Nous sommes les dépôts</h3>


<p><strong>À partir de maintenant, pensez à votre application comme étant rien d'autre qu'un dépôt à ressources.</strong> Qu'est-ce qu'un dépôt&nbsp;? <a href="https://larlet.fr/david/biologeek/archives/20060707-installer-un-depot-subversion-sous-ubuntu/">Subversion</a> est un bon exemple de dépôt de code source. MySQL est un dépôt de données relationnelles. Vous pensez à d'autres exemples&nbsp;?</p>


<p>Pour le moment, j'ai juste besoin de prendre un exemple de dépôt que l'on puisse utiliser dans la suite de ce billet.</p>


<h3>Le pire exemple au monde</h3>


<p>Vous travaillez pour BiologeekAir, et vous devez développer leur infrastructure logicielle <em>from scratch</em>.</p>

<ul>
<li>Vous avez besoin d'un client riche, une interface permettant de saisir les informations de vol. Un vol est juste la définition d'un avion allant d'une ville à une autre à un moment donné.</li>
<li>Vous voulez une interface web pour les employés de l'aéroport. Vous voulez que les passagers puissent vérifier le statut du vol à partir de leur téléphone portable.</li>
<li>Et votre boss veut que vous créiez une belle API basée sur XML que les tiers pourront utiliser pour mettre à jour les gigantesques affichages de départs et d'arrivées de l'aéroport.</li>
</ul>

<p>Ça semble être un projet alléchant, non&nbsp;? Souvenez-vous de la première partie, nous avons décrit une architecture orientée objet traditionnelle et la façon dont il faut commencer en décrivant votre problème pour identifier les noms (qui deviendront vos objets) et les verbes (qui deviendront les méthodes). Imaginez juste le nombre de design patterns géniaux que vous allez pouvoir mettre en œuvre&nbsp;! Vous ne pouvez plus résister à l'envie de dessiner un énorme diagramme UML au tableau et d'enchaîner avec quelques diagrammes d'interactions&nbsp;! Oh, vous imaginez déjà les méthodes VolsPrévus.AnnulerVol() et Vol.Retard().</p>


<p>Désolé David. <strong>Depuis que vous utilisez REST, une grande partie de votre design est déjà fait pour vous&nbsp;! En fait, la seule chose qu'il vous reste à faire est d'identifier vos ressources.</strong></p>


<p>Laissez-moi le répéter, au cas où vous ne m'auriez pas entendu avec le raffut que font vos marqueurs effaçables, car c'est vraiment important.</p>


<p><strong>Ne faites qu'identifier vos ressources.</strong></p>


<p>Dans notre exemple, je peux rapidement identifier quelques noms clés&nbsp;: avions, aéroports et vols. Je vais probablement en trouver davantage ensuite mais c'est déjà un bon début.</p>


<p>Comment les interfaces clients vont-elles accéder à chacune de ces ressources&nbsp;? Les URL ont été inventées pour ça. (Vous savez ce que signifie le L de <abbr title="Uniform Resource Locator">URL</abbr> n'est ce pas ?) Le «&nbsp;localisateur » de chaque ressource suit un pattern simple. Par exemple&nbsp;:</p>

<ul>
<li>/aeroports pour la liste des aéroports</li>
<li>/avions pour la liste des avions</li>
<li>/vols pour la liste des vols</li>
</ul>

<p>et&nbsp;:</p>

<ul>
<li>/aeroports/ory pour Paris Orly</li>
<li>/avions/ZJ3543 pour l'avion identifié par le numéro de série ZJ3543</li>
<li>/vols/451 pour le vol #451</li>
</ul>

<h3>Quatre méthodes</h3>


<p>Ok, une fois vos ressources identifiées, REST définie les 4 méthodes (ou verbes en langage HTTP) que chacune de vos ressources peuvent implémenter&nbsp;:</p>

<ul>
<li><strong>GET&nbsp;: donnez moi une vue en lecture seule d'une instance d'une ressource depuis le dépôt.</strong></li>
<li><strong>POST&nbsp;: j'ai une nouvelle instance d'une ressource que je veux ajouter au dépôt.</strong></li>
<li><strong>PUT&nbsp;: je veux mettre à jour une ressource existante dans le dépôt.</strong></li>
<li><strong>DELETE&nbsp;: je veux supprimer une instance d'une ressource du dépôt.</strong></li>
</ul>

<p>Ça vous semble familier&nbsp;? Avec Subversion, GET est similaire aux commandes «&nbsp;checkout » ou «&nbsp;update ». POST correspond à la commande «&nbsp;add » et DELETE à «&nbsp;delete ». Dans l'exemple avec MySQL, GET revient à faire un SELECT, POST un INSERT, PUT est semblable à UPDATE et DELETE, et bien, DELETE.</p>


<p>Wow. Nous avons nos ressources et nous avons nos méthodes. On peut arrêter ici le design et commencer l'implémentation.</p>


<h3>Appeler ces méthodes avec HTTP</h3>


<p>Les applications web utilisent HTTP pour permettre au serveur et au client de communiquer ensemble. On peut maintenant préciser la première partie qui était restée assez floue. Que se passe-t-il vraiment lorsque vous tapez une adresse dans votre navigateur&nbsp;? Le navigateur met en forme un message HTTP pour appeler la méthode GET de la ressource identifiée par l'URL que vous lui avez fourni.</p>


<p>Vous pouvez aussi appeler la méthode POST à partir de votre navigateur - mais seulement s'il y a un formulaire &lt;form&gt; sur la page. (Ok, pas exactement grâce à AJAX mais oublions pour le moment). Les formulaires peuvent envoyer des données à une URL particulière. Lorsque vous soumettez un formulaire, le navigateur appelle la méthode POST, passant l'URL ainsi que certaines données représentant la ressource qu'il doit ajouter ou modifier.</p>


<p><strong>Vous ne pouvez pas appeler PUT ou DELETE avec votre navigateur avec du HTML habituel</strong>. Ce qui est bien malheureux. Bien que chaque serveur HTTP dans le monde sache gérer les appels aux méthodes PUT et DELETE, il n'y a pas de procédure standardisée pour faire ces appels avec du HTML. Vous devez inventer votre propre façon de faire pour signaler au serveur que vous essayer de faire des appels à PUT ou DELETE.</p>


<p>En résumé&nbsp;: <strong>vous pouvez faire correspondre votre API REST avec les verbes HTTP mais il n'y a aucun tag HTML qui permette de faire des appels à PUT ou DELETE</strong>. Le client et le serveur doivent se mettre d'accord sur une convention à utiliser pour simuler les méthodes PUT et DELETE.</p>


<h3>On y est presque</h3>


<p>Maintenant que notre design est au point, nous l'implémenterons prochainement avec <del>Rails</del> Django ;-).</p>


<h2>Réactions du traducteur</h2>


<p>Bon vous l'aurez compris, les prochains billets ne seront pas des traductions d'exemples d'implémentation avec Ruby on Rails mais des articles originaux pour voir ce qu'il est possible de faire avec Django. Plusieurs projets ont vu le jour et vont tenter de s'unifier pour arriver à une solution cohérente et facilement intégrable. Bon accessoirement, j'aimerais bien mettre en place professionnellement une telle architecture donc ça m'intéresse beaucoup.</p>


<p>En ce qui concerne <strong>REST, sa principale limitation vient de la non standardisation des méthodes PUT et DELETE dans les navigateurs</strong>. Je pense que c'est l'un des freins actuel à son adoption face à d'autres alternatives. Néanmoins, la possibilité de mettre en place du REST facilement côté serveur avec la dernière version de Ruby on Rails va, je l'espère, un peu démocratiser tout ça. Corrigez-moi si je me trompe mais RoR a choisi de placer une champ caché avec l'attribut <strong>method="PUT"</strong> (ou DELETE) pour gérer les différentes possibilités. Espérons que de telles pratiques deviennent la norme de façon à ce que <strong>REST devienne réellement la référence en terme d'interopérabilité entre les différents services web</strong>.</p>


<h2>Ressources</h2>


<h3>En français</h3>

<ul>
<li><a href="http://www.la-grange.net/2005/05/rest/">Apprendre REST - un style d'architecture du Web</a></li>
<li><a href="http://www.clever-age.com/veille/clever-link/soap-vs.-rest-choisir-la-bonne-architecture-web-services.html">soap vs. rest&nbsp;: choisir la bonne architecture web services</a></li>
<li><a href="http://opikanoba.org/tr/fielding/rest/">Traduction du chapitre de la Thèse de Roy T. Fielding relatif à REST</a></li>
</ul>

<h3>En anglais</h3>

<ul>
<li><a href="http://tomayko.com/articles/2004/12/12/rest-to-my-wife">How I Explained REST to My Wife</a> (et <a href="http://www.pompage.net/pompe/comment-j-ai-explique-rest-a-ma-femme/">maintenant traduit en français</a>&nbsp;! Merci Karl).</li>
<li><a href="http://duncan-cragg.org/blog/tag/dialogue/">The REST Dialogues</a>&nbsp;: c'est un catégorie de son blog, il faut donc commencer par le dernier, c'est long mais ça en vaut la peine, surtout qu'il y a une <a href="http://addsimplicity.typepad.com/adding_simplicity_an_engi/2006/11/the_rest_dialog.html">vraie réponse d'un architecte eBay</a> qui a joué le jeu (il a répondu à d'autres aussi, généralement le lien est en commentaire).</li>
</ul>

<p>Je n'en mets pas plus car vous allez très vite être submergé de liens, c'est juste pour vous mettre en appétit, j'espère que vous êtes maintenant convaincu que <strong>REST est plus qu'une alternative à SOAP</strong>&nbsp;!</p>


<p><strong>[edit]</strong>&nbsp;: un nouveau billet intitulé <a href="https://larlet.fr/david/biologeek/archives/20070629-architecture-orientee-ressource-pour-faire-des-services-web-restful/">L'architecture orientée ressource pour faire des services web RESTful</a> précise de nombreux points sur l'architecture REST non couverts dans ce billet.</p>
