title: Principes Web
slug: principes-web
date: 2015-07-28
chapo: Les principes qui m’animent lorsque je conçois et réalise un site Web.

Cette histoire [d’Appel d’Offre](/david/blog/2015/coconstruction-ao/) m’a amené à réfléchir aux principes qui m’animent lorsque je conçois et réalise un site Web. Même s’il y a du recouvrement, je les considère comme étant orthogonaux au [manifeste de développeur](/david/blog/2014/manifeste-developpeur/) précédemment rédigé. Ces principes vont certainement servir de fil rouge aux [cours de cette année](/david/pro/enseignement/) donc vos retours sont les bienvenus.

## Accessibilité

> **Responsive web design is all about accessibility.** Although it is hard to make a business case for it, accessibility is very important, as it is a core concept of the web, as Aaron mentioned, and, being mostly device-agnostic, our best bet at future proofing content for screens that don’t even exist yet, as Rosie pointed out.
> 
> <cite>*[Responsive day out 3: the final breakpoint](https://hiddedevries.nl/en/blog/2015-06-20-responsive-day-out-3-the-final-breakpoint/)* ([cache](/david/cache/b81e5ead29fc9dcca90932464e54c8ed/))</cite>

J’ai longtemps cru que l’accessibilité était dédiée aux personnes présentant des déficiences. *J’avais tort.* Avoir un site accessible ne signifie pas être conforme au [RGAA](https://references.modernisation.gouv.fr/rgaa-3-0) ou être lisible par un lecteur d’écran. Avoir un site accessible c’est le rendre [disponible](http://timkadlec.com/2015/06/thriving-in-unpredictability/) ([cache](/david/cache/3f9bce9494214d3a3f21dd05ebab85f3/)) par le plus grand nombre en améliorant le rapport signal/bruit et la lisibilité, c’est le rendre accessible quel que soit le périphérique autant dans sa résolution que dans sa puissance ou sa connectivité, c’est le rendre sauvegardable/liable/partageable, c’est [l’améliorer progressivement](http://sixtwothree.org/posts/designing-with-progressive-enhancement) ([cache](/david/cache/dc941db5d707f2708939323d9cd93442/)) et incrémentalement. **L’accessibilité est la base de la pyramide de Maslow de l’expérience utilisateur.**

Vous pouvez ajouter toute la *gamification*, le *growthhacking* ou l’intégration aux réseaux sociaux que vous voulez après ça, si vous n’avez pas une base solide à votre pyramide ça ne sert à rien. Les visiteurs seront partis bien avant d’avoir eu la moindre interaction avec vous. En produisant de la qualité lisible (et indexable) par tous vous êtes déjà dans l’infime minorité des sites qui donnent envie d’aller plus loin avec vous sur le net.

## Pérennité

> We, as a species, are currently putting together a universal repository of knowledge and ideas, unprecedented in scope and scale. Which information-handling technology should we model it on? The one that’s worked for 4 billion years and is responsible for our existence? Or the one that’s led to the greatest intellectual tragedies in history?
>
> <cite>*[The Web of Alexandria](http://worrydream.com/TheWebOfAlexandria/)* ([cache](/david/cache/bd244789119bc6d32deb987b95e37043/))</cite>

Chaque produit doit se questionner sur la pérennité de ses données et de ses liens. Parfois c’est futile et temporaire, d’autres fois c’est jugé comme étant une ressource clé qui doit être conservée une décennie ou deux. Cette contrainte doit s’envisager de manière stratégique. Les technologies employées importent peu, il s’agit avant tout de garder le contrôle sur les données et les liens qui sont fait.

Il y a deux sources possibles d’endommagement de votre capital :

* vous et les parties prenantes du projet ;
* les attaques externes.

Les secondes sont bien plus faciles à gérer :-). Ne négligez pas la menace interne — humaine — en commençant par vous. Éprouvez vos sauvegardes et vos stratégies de migration. **Passez plus de temps à documenter vos données qu’à documenter votre code.** Évaluez ce qui est critique en terme de confidentialité et de confiance pour vos utilisateurs, *avec vos utilisateurs* et limitez leur périmètre.

## Frugalité

> Datensparsamkeit is a concept from these privacy laws that is an opposite philosophy to "capture-all-the-things". A translation isn't straightforward (which is why I've retained the German word) but loosely you might translate it as something like "data austerity", "data minimization", "data parsimony", or "data frugality". It means that you should always ask yourself why you are capturing or storing data, and look to handle only the minimum amount of data you need for your purpose.
>
> <cite>*[Datensparsamkeit](http://martinfowler.com/bliki/Datensparsamkeit.html)* ([cache](/david/cache/2d7e6b8ac2e7f33001a34f1884f00d6c/))</cite>

Récolter toutes les données possibles est la solution de facilité. Il faut remettre en cause [chaque champ de formulaire](http://www.nngroup.com/articles/mobile-input-checklist/) ([cache](/david/cache/7fe9e2c3bafffac17d376f359ab9c323/)), chaque donnée personnelle collectée. S’interroger sur les lois auxquelles elles sont soumises et [les solutions à mettre en place](http://idlewords.com/bt14.htm) ([cache](/david/cache/bf91cb0067cdafb97553ec195630fc76/)) pour informer vos utilisateurs. **Toute donnée non collectée est une fuite évitée.**

Il en est de même pour les données servies au visiteur, accédez à votre site depuis un TGV et [constatez facilement les dégâts](https://medium.com/@zengabor/three-takeaways-for-web-developers-after-two-weeks-of-painfully-slow-internet-9e7f6d47726e) ([cache](/david/cache/e00ae9f67fe78b3490ecafe16a7802d4/)). Les performances de votre site sont directement affectées par votre gourmandise. Il faut remettre en question chaque dépendance (externe), les publicités et les statistiques ne sont peut-être pas essentielles pour vos utilisateurs et/ou la survie du service.

## Empathie

> * **Early customer validation** over releasing products with unknown end-user value
> * **Collaborative design** over designing on an island
> * **Solving user problems** over designing the next “cool” feature
> * **Measuring KPIs** over undefined success metrics
> * **Applying appropriate tools** over following a rigid plan
> * **Nimble design** over heavy wireframes, comps or specs
>
> <cite>*[The Lean UX Manifesto: Principle-Driven Design](http://www.smashingmagazine.com/2014/01/08/lean-ux-manifesto-principle-driven-design/)* ([cache](/david/cache/178740c41843109b2d3034da74746315/))</cite>

La transformation d’un produit utile en [un produit « charmant »](https://medium.com/the-happy-startup-school/beyond-mvp-10-steps-to-make-your-product-minimum-loveable-51800164ae0c) ([cache](/david/cache/a1bf79316204474c8d8522758f017124/)) est très difficile. C’est pourtant celle qui fait la différence, trop de produits *Open-Source* sont malheureusement là pour le démontrer… **Pour aller de l’efficace à l’enviable, il faut avant tout avoir une approche basée sur l’écoute et la remise en question.** Limiter les contraintes et réduire la pression cognitive d’interaction avec vos contenus.

[Penser son produit](https://medium.com/@jaf_designer/why-product-thinking-is-the-next-big-thing-in-ux-design-ee7de959f3fe) ([cache](/david/cache/42b387daa6d294c18983ebb91a6c29c5/)) va au-delà de l’esthétique ou des interactions, il s’agit d’accompagner l’utilisateur dans son apprentissage d’un service ainsi que dans son amélioration.  La confiance qui doit s’établir pour qu’un produit progresse réclame du temps et de l’attention.

## Chemin

> This switch from web design to experience design is directly caused by the shift from web pages to digital products, tools, and ecosystems. Web pages are just part of something much bigger: mobile apps, API’s, social media presence, search engine optimization, customer service channels, and physical locations all inform the experience a user has with a brand, product, or service. Pretending that you can run a business or deliver value just by taking care of the web channel is naïve at best and harmful at worst.
>
> <cite>*[Web design is dead](http://mashable.com/2015/07/06/why-web-design-dead/)* ([cache](/david/cache/cfea7986147a83150a85fb636d396a99/))</cite>

Une présence en ligne n’est plus un but, c’est un chemin. **C’est une relation tripartite entre ceux qui cultivent, ceux qui cuisinent et ceux qui agencent le jardin.** Il y a des saisonnalités et des objectifs intermédiaires mais il n’y a pas d’arrêt dans ce cycle. C’est la raison pour laquelle [les devis sont désuets](/david/pro/devis/) (et les Appels d’Offres également).

Si le [Web est un *continuum*](https://adactio.com/journal/6692) ([cache](/david/cache/1efa2e01199c9e2e3f761386162bf875/)), la relation qui permet de le produire l’est aussi. Trop [de produits échouent](http://www.svpg.com/product-fail) ([cache](/david/cache/062df59908e9101831be29fd1235e170/)) faute de communication à la fois dans l’équipe mais aussi avec les utilisateurs. Pour la communication interne, les approches agiles donnent des outils, pour la communication externe, le *LEAN* donne des pistes avec les interviews et la conception de [produits essentiels](/david/pro/produits-essentiels/).

***

Ces principes guident mes choix lorsque j’interviens sur un projet, ils permettent d’établir une vision stratégique sur le long terme du produit et/ou d’identifier les points bloquants spécifiques à traiter de manière tactique et ponctuelle. Ils constituent une base de discussion et de remise en question personnelle continue.
