title: Web et technique
slug: web-technique
date: 2017-07-21
chapo: Le pouvoir technique, c’est le monopole de la dette légitime.

> L’évolution des objets techniques doit être comprise en ce sens comme la recherche d’un système qui est tel qu’il ne peut être autodestructif. Le système doit pouvoir se maintenir stable le plus longtemps possible. Pour le dire autrement, il s’agit au fond d’augmenter l’espérance de vie de l’objet technique — ce que rend précisément possible l’établissement de directions de convergence. Un objet technique est d’autant plus évolué qu’il n’est pas en lutte avec lui-même. Un objet technique évolué est celui dans lequel aucun effet secondaire ne nuit au fonctionnement de l’ensemble, ou aucun effet secondaire n’est laissé en dehors de ce fonctionnement. Un objet technique se caractérise par le fait de son inscription au sein d’une lignée technique qui augmente à chaque génération son espérance de vie par l’artifice d’une auto-corrélation.
> 
> <cite>*[Gilbert Simondon et la libération par les techniques](https://www.slate.fr/tribune/85761/gilbert-simondon-liberation-par-les-techniques)* ([cache](/david/cache/18a83df62e9855e9d8c98f6b730620a3/))</cite>

Difficile de ne pas faire un parallèle avec le Web ici, objet technique en lutte permanente avec lui-même. La question est de savoir s’il y a effectivement convergence et augmentation de son espérance de vie au cours du temps. J’en doute de plus en plus. Et de continuer :

> Mais, précisément, est-ce bien un artifice ? L’objet technique est-il artificiel ? Ce qui est frappant, bien plutôt, note Simondon, est que l’objet technique évolué se rapproche du mode d’existence des objets naturels, en ce sens où il tend lui aussi vers la cohérence interne ou la fermeture du système des causes et des effets qui s’exercent circulairement au sein de son enceinte. On estime d’ordinaire que ce qui fait l’artificialité d’un objet ne tient pas seulement à ceci que l’objet considéré a été fabriqué (et non pas produit spontanément par la nature), mais aussi et surtout à ceci que l’intervention de l’homme est nécessaire pour maintenir cet objet dans l’existence en le protégeant contre le monde naturel, en lui donnant ainsi un statut à part d’existence. L’artificialité est ce qui est intérieur à l’action artificialisante de l’homme : est artificiel ce qui requiert le concours de l’homme non seulement pour exister, mais encore pour se maintenir dans l’existence. Or le propre d’un objet technique est justement qu’il requiert de moins en moins l’intervention de l’homme pour se maintenir dans l’existence : c’est un objet artificiel (en tant qu’objet fabriqué) qui a un mode d’existence naturel.
>
> <cite>*Ibid.*</cite>

Je ne peux m’empêcher de faire le parallèle ici encore avec un Web de plus en plus artificiel de par son [manque de résilience](/david/blog/2017/resilient-web-tools/), son [rythme effréné](https://www.offscreenmag.com/blog/the-grip-of-now) ([cache](/david/cache/b53e9d9d0598024ba2524f6dd74f1804/)) et [l’usage abscons de JavaScript](/david/blog/2017/without-javascript/). Ce qui m’amène à penser à l’autonomie du code. Si vous ne touchez plus au code de votre produit Web actuel pendant 6 mois, combien de journées seront nécessaires pour le remettre à jour ? **Mettre le doigt sur les causes de son artificialité permet de participer à l’évolution d’un outil technique.** De faciliter son émancipation en réduisant sa [dépendance aux développeurs](/david/blog/2016/simplicite-defaut/) tout en [maximisant les problèmes résolus](https://www.ultrasaurus.com/2017/06/customer-driven-development/) ([cache](/david/cache/c5ae918d293c87be86d7d9070dec9629/)). 

Pour paraphraser Max Weber au sujet de l’État qui considérait que <em>« le pouvoir politique, c’est le monopole de la violence légitime »</em> on aurait <em>« le pouvoir technique, c’est le monopole de la dette légitime »</em>. Or qui introduit cette dette aujourd’hui ? Les GAFAM plus que l’État à travers des bibliothèques populaires (React, Angular) qui évoluent très rapidement. Cette dette ne s’arrête pas aux outils mais aussi à leurs usages : en utilisant gratuitement un produit on en devient implicitement redevable.

Est-ce que le logiciel libre est ce « mode d’existence naturel » ? En un sens oui, car il devient théoriquement entretenu par une communauté qui se renouvelle et le fait évoluer itérativement. En pratique malheureusement on est bien loin du compte, il y aurait probablement à piocher dans des principes éloignés du code pour parvenir non pas à la [convivialité](/david/blog/2013/open-source-conviviale/) mais peut-être à la résilience :

> * **Homéostasie** : des boucles multiples de rétroaction pour contrer les perturbations et stabiliser le système ; l'homéostasie est la capacité que peut avoir un système quelconque à conserver son équilibre de fonctionnement en dépit des contraintes qui lui sont extérieures.
> * **Omnivore** : la vulnérabilité est réduite par la diversification des ressources et des moyens ; comme dans la nature l'ultra spécialisation entame les potentiels de survie.
> * **Flux rapides** : des mouvements des ressources rapides à travers le système assurent la mobilisation de ces ressources pour faire face aux perturbations.
> * **Niveaux hiérarchiques faibles** : afin de mettre en œuvre rapidement des réponses très locales non standard. Comme chez nos amis les fourmis ou les rats.
> * **Capacité tampon** : capacités centrales sur-dimensionnées de telle sorte que les seuils critiques soient moins susceptibles d’être franchis. C'est ce qui est à l’œuvre dans les végétaux du désert et leur capacité interne en eau.
> * **Redondance** : les fonctions se chevauchent, et un relais peut ainsi être assuré si certaines échouent. Les espèces d’animaux redondantes sont celles qui exercent la même fonction au sein de l’écosystème. Comme les vautours et les hyènes dont les fonctions de découpage des animaux morts sont une même fonction pour deux espèces.
>
> <cite>*[Résilience et modularité](https://nicolasloubet.gitbooks.io/fork-the-world/content/chapter4/resilience-et-modularite.html)* ([cache](/david/cache/1df55eefb86dd35c0e758fac873bab0a/)) — Source : [PDF en anglais](http://www.nusap.net/downloads/Wardekker_ea_2010_TFS.pdf) ([cache](/static/david/blog/Wardekker_ea_2010_TFS.pdf))</cite>

Il s’agirait presque de la définition d’un réseau en pair-à-pair comme [SSB](/david/blog/2017/confiance-transparence/). Notez qu’il n’y a pas de référence à la croissance, un écosystème ayant besoin de s’auto-équilibrer pour survivre.

*Quelques pensées (techniques ?) mal agencées que je vous laisse maintenir en vie :-).*