title: Python et namedtuples

> Many functional programming articles teach abstract functional techniques. That is, composition, pipelining, higher order functions. This one is different. It shows examples of imperative, unfunctional code that people write every day and translates these examples to a functional style.
> 
> <cite>*[A practical introduction to functional programming](http://maryrosecook.com/blog/post/a-practical-introduction-to-functional-programming)* ([cache](/david/cache/437dce93077f9a2569566bfcfe47134d/))</cite>

Dans les épisodes précédents, on a vu comment [récupérer des billets voisins](/david/stream/2015/01/25/) et [parcourir des fichiers markdown](/david/stream/2015/02/03/). Il est temps de combiner les deux. Mais avant cela, commençons par gérer des objets par l’intermédiaire des `namedtuples` qui facilitent l’accès aux attributs :

	:::python
    Post = namedtuple('Post', ['content', 'author', 'date'])
	def populate_post(content, metadata):
		# Divers traitements.
		return Post(title=title, date=date_, content=content)

Cette méthode va nous permettre de générer des `posts` à partir du contenu et des métadonnées de notre fichier *markdown*. On va ensuite vouloir récupérer une liste ordonnée par date :

    :::python
    def collection():
        """Retrieve all posts sorted by date desc."""
        posts = []
        for file_path in each_markdown_from(POSTS_PATH):
            content, metadata = parse_markdown(file_path)
            post = populate_post(content, metadata)
            posts.append(post)
        return sorted(posts, key=attrgetter('date'), reverse=True)

C’est cette liste dont on peut utiliser les voisins pour la génération du fichier `HTML` final :

    :::python
    for previous, post, next in neighborhood(collection(), last=thoughts):
        # On peut ici accéder à post.title, post.content, etc.

J’aurais pu rendre tout cela encore plus fonctionnel mais je préfère pouvoir vérifier facilement les étapes intermédiaires sans qu’il y ait trop d’imbrications incompréhensibles. C’est la fin de cette série, vous avez maintenant les outils pour créer votre propre moteur de blog statique !

